function [output] =  stop_foforth_manopt2stop(problem, Xinit, options)

manifold_name = problem.M.name();
% manifold_name(1:10)
% strcmp(manifold_name(1:16), 'Stiefel manifold')
if ~strcmp(manifold_name(1:16), 'Stiefel manifold') && ~strcmp(manifold_name(1:18), 'Grassmann manifold')
    error('Wrong mmanifold. This warpper only receives stiefel manifold or Grassmannian manifold.')
end

if ~exist('Xinit', 'var') || isempty(Xinit)
        Xinit = problem.M.rand();
end

if ~exist('options', 'var') || isempty(options)
        options = struct();
end


opts = manopt2stop_opts(options);

if ~isfield(opts, 'X')
    opts.X = Xinit;
end

obj_fun = @(X) manopt2stop_CostGrad(problem, X);

[output] = stop_foforth(obj_fun, opts);
end


function opts = manopt2stop_opts(options)

    opts = options;
    if isfield(options, 'tolgradnorm')
        opts.gtol = options.tolgradnorm;
    end
    
    if isfield(options, 'maxiter')
        opts.maxit = options.maxiter;
    end
    
    if isfield(options, 'lambdamax')
        opts.stepsize.max_stepsize = options.lambdamax;
    end
    
    if isfield(options, 'verbosity')
        opts.info = options.verbosity;
    end
    
   
end



function [cost, grad] = manopt2stop_CostGrad(problem, x)
% cost \& grad should not contains key

    storedb = StoreDB();
    key = storedb.getNewKey();
    store = storedb.getWithShared(key);
    if isfield(problem, 'costgrad')
    %% Compute the cost/grad pair using costgrad.
    
        % Check whether this function wants to deal with storedb or not.
        switch nargin(problem.costgrad)
            case 1
                [cost, grad] = problem.costgrad(x);
            case 2
                [cost, grad, store] = problem.costgrad(x, store);
                
                
            case 3
                % Pass along the whole storedb (by reference), with key.
                [cost, grad] = problem.costgrad(x, storedb, key);
        end

    else
    %% Revert to calling getCost and getGradient separately
    
        % The two following calls will already cache cost and grad, then
        % the caches will be overwritten at the end of this function, with
        % the same values (it is not a problem).
        cost = getCost(problem, x, storedb, key);
        grad = getGradient(problem, x, storedb, key);
        
    end
    
end