# STOP -- a toolbox for STiefel manifold OPtimization

[STOP](https://stmopt.gitee.io) is a toolbox developed for solving the optimization problems with orthogonality constraints.

$$
\begin{equation*}
  \begin{aligned}
    \min\limits_{X \in \mathbb{R}^{n \times p}} \hspace{2mm} & f (X)  \\
    \mathrm{s.t.} \hspace{3mm} & X \in \mathcal{S}_{n, p} := \{X \in \mathbb{R}^{n \times p} \mid X^{\top} X = I_p \}.
  \end{aligned}
\end{equation*}
$$

Here, $f(X): \mathbb{R}^{n \times p} \to \mathbb{R}$ is an objective function, 
and the feasible set $\mathcal{S}_{n, p}$ is usually referred to as the Stiefel manifold.

There are two categories of solvers in STOP, including **feasible** methods and **infeasible** methods. Distinct from Riemannian optimization algorithms, STOP captures the special structures of Stiefel manifolds, and hence can achieve high efficiency.
Unless otherwise specified, for a smooth function $g(X)$, $\nabla g(X)$ and $\nabla^2 g(X)$ stand for the **Euclidean** gradient and Hessian in STOP, respectively.

More information can be found in the [Users’ Guide](https://stmopt.gitee.io/tutorial.html).


## Feasible Methods

STOP provides a class of feasible methods, called [multipliers correction methods](https://stmopt.gitee.io/MCM.html).

Multipliers correction methods aim to minimize a smooth function over the Stiefel manifold, which combine a **function value reduction step** with a **correction step**. The former one searches along the standard Euclidean descent directions instead of the vectors in the tangent space of the Stiefel manifold, and the latter one further reduces the function value and guarantees a symmetric dual variable at the same time.

- FOForth focuses on the structured objective functions $f (X) = h (X) + \mathrm{tr} (G^{\top} X)$.
  - $G \in \mathbb{R}^{n \times p}$ is a constant matrix.
  - $h (X)$ is invariant under the orthogonal transformation, i.e., $h(XQ) = h(X)$ holds for any $Q \in \mathcal{S}_{p,p}$. 
  - $\nabla h(X) = H(X) X$, where $H:\mathbb{R}^{n \times p} \to \mathbb{S}^{n}$ and $\mathbb{S}^{n}$ refers to the set of $n \times n$ symmetric matrices. 

- ProxOrth extends the framework of FOForth to generic smooth objectives.

## Infeasible Methods
The orthonormalization process has low parallel scalability. With the increasing of $p$, a bottleneck of existent algorithms emerges, that is, *lack of concurrency*. To address this issue, STOP provides a class of infeasible methods to waive the orthonormalization process. Hence, parallelization becomes tractable in solving the optimization problems over the Stiefel manifold.

- [For smooth objectives](https://stmopt.gitee.io/Smooth.html).
  - PCAL is based on the augmented Lagrangian method (ALM), but updates the multipliers by a closed-form expression.
  - PenCF uses a first-order approach to solve an exact penalty model.  
  - PenCS uses a second-order approach to solve an exact penalty model.  

- [For nonsmooth objectives](https://stmopt.gitee.io/Nonsmooth.html).
  - PenCPG minimizes a smooth function plus an $\ell_{2,1}$-norm regularizer over the Stiefel manifold. It uses a proximal gradient approach to solve an exact penalty model.  
  - SLPG alternately takes tangential steps and normal steps to improve the optimality and feasibility respectively, which is penalty-free. 

## References
If you use these codes in an academic paper, please cite the following papers.

1. B. Gao, X. Liu, X. Chen, and Y. Yuan, [*A New First-order Algorithmic Framework for Optimization Problems with Orthogonality Constraints*](https://epubs.siam.org/doi/abs/10.1137/16M1098759), SIAM Journal on Optimization, 28-1(2018), 302–332.
2. L. Wang, B. Gao, and X. Liu, [*Multipliers Correction Methods for Optimization Problems over the Stiefel Manifold*](https://www.global-sci.org/intro/article_detail/csiam-am/19448.html), CSIAM Transactions on Applied Mathematics, 2-3(2021), 508-531.
3. B. Gao, X. Liu, and Y. Yuan, [*Parallelizable Algorithms for Optimization Problems with Orthogonality Constraints*](https://epubs.siam.org/doi/abs/10.1137/18M1221679#), SIAM Journal on Scientific Computing, 41-3(2019), A1949–A1983.
4. N. Xiao, X. Liu, and Y. Yuan, [*A Class of Smooth Exact Penalty Function Methods for Optimization Problems with Orthogonality Constraints*](https://www.tandfonline.com/doi/full/10.1080/10556788.2020.1852236?journalCode=goms20), Optimization Methods and Software. 
5. N. Xiao, X. Liu, and Y. Yuan, [*Exact Penalty Function for $\ell_{2,1}$ Norm Minimization over the Stiefel Manifold*](http://www.optimization-online.org/DB_HTML/2020/07/7908.html), SIAM Journal on Optimization.
6. N. Xiao, X. Liu, and Y. Yuan, [*A Penalty-free Infeasible Approach for a Class of Nonsmooth Optimization Problems over the Stiefel Manifold*](https://arxiv.org/abs/2103.03514), arXiv:2103.03514.

## Authors
STOP is developed by the following authors listed in alphabetical order in the [State Key Laboratory of Scientific and Engineering Computing](http://lsec.cc.ac.cn), [Institute of Computational Mathematics and Scientific/Engineering Computing](http://www.cc.ac.cn), [Academy of Mathematics and Systems Science](http://www.amss.cas.cn), [Chinese Academy of Sciences](http://english.cas.cn).
- [Bin Gao](https://www.gaobin.cc/) (gaobin@lsec.cc.ac.cn)
- [Xin Liu](http://lsec.cc.ac.cn/~liuxin/index.html) (liuxin@lsec.cc.ac.cn)
- [Lei Wang](https://orcid.org/0000-0003-0651-6262) (wlkings@lsec.cc.ac.cn)
- [Nachuan Xiao](https://xnchxy.github.io/) (xnc@lsec.cc.ac.cn)
- [Ya-xiang Yuan](http://lsec.cc.ac.cn/~yyx/) (yyx@lsec.cc.ac.cn)


## To Contact Us
If you have any questions or find any bugs, please feel free to contact us by email (stmopt@foxmail.com).

In oreder to help you use STOP, we are happy to receive feedbacks, bug reports and requests for more features, and to discuss the toolbox as well as its documentation. We would also like to know how you use the toolbox.


## License
STOP is a free software. You can redistribute it and/or modify it under the terms of the [GNU Lesser General Public License](https://www.gnu.org/licenses/lgpl-3.0.en.html) as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

STOP is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Please see the [GNU Lesser General Public License](https://www.gnu.org/licenses/lgpl-3.0.en.html) for more details.