function [output] =  SLPG_smooth_manopt2stop(problem, Xinit, options)

if ~exist('Xinit', 'var') || isempty(Xinit)
        Xinit = problem.M.rand();
end

if ~exist('options', 'var') || isempty(options)
        options = struct();
    end


opts = manopt2stop_opts(options);
opts.X = Xinit;

obj_fun = @(X) manopt2stop_CostGrad(problem, X);

[output] = stop_slpg_smooth(obj_fun,opts);
end



function [cost, grad] = manopt2stop_CostGrad(problem, x)
% cost \& grad should not contains key

    storedb = StoreDB();
    key = storedb.getNewKey();
    store = storedb.getWithShared(key);
    if isfield(problem, 'costgrad')
    %% Compute the cost/grad pair using costgrad.
    
        % Check whether this function wants to deal with storedb or not.
        switch nargin(problem.costgrad)
            case 1
                [cost, grad] = problem.costgrad(x);
            case 2
                 [cost, grad, store] = problem.costgrad(x, store);
                
                
            case 3
                % Pass along the whole storedb (by reference), with key.
                [cost, grad] = problem.costgrad(x, storedb, key);
        end

    else
    %% Revert to calling getCost and getGradient separately
    
        % The two following calls will already cache cost and grad, then
        % the caches will be overwritten at the end of this function, with
        % the same values (it is not a problem).
        cost = getCost(problem, x, storedb, key);
        grad = getGradient(problem, x, storedb, key);
        
    end
end


function opts = manopt2stop_opts(options)

    opts = options;
    if isfield(options,'tolgradnorm')
        opts.gtol = options.tolgradnorm;
    end
    
    if isfield(options,'maxiter')
        opts.maxit = options.maxiter;
    end
    
    if isfield(options,'lambdamax')
        opts.stepsize.max_stepsize = options.lambdamax;
    end
    
    if isfield(options,'verbosity')
        opts.info = options.verbosity;
    end
end
